@@ -3,7 +3,7 @@ ldapobject.py - wraps class _ldap.LDAPObject
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapobject.py,v 1.103 2009/07/26 11:09:58 stroeder Exp $
+\$Id: ldapobject.py,v 1.105 2009/10/23 09:17:37 stroeder Exp $
 
 Compability:
 - Tested with Python 2.0+ but should work with Python 1.5.x
@@ -94,15 +94,23 @@ class SimpleLDAPObject:
     try:
       try:
         result = func(*args,**kwargs)
+        if __debug__ and self._trace_level>=2:
+          if func.__name__!="unbind_ext":
+            diagnostic_message_success = self._l.get_option(ldap.OPT_DIAGNOSTIC_MESSAGE)
+          else:
+            diagnostic_message_success = None
       finally:
         self._ldap_object_lock.release()
     except LDAPError,e:
       if __debug__ and self._trace_level>=2:
         self._trace_file.write('=> LDAPError - %s: %s\n' % (e.__class__.__name__,str(e)))
       raise
-    if __debug__ and self._trace_level>=1:
-      if self._trace_level>=2 and result!=None and result!=(None,None):
-        self._trace_file.write('=> result: %s\n' % (repr(result)))
+    else:
+      if __debug__ and self._trace_level>=2:
+        if not diagnostic_message_success is None:
+          self._trace_file.write('=> diagnosticMessage: %s\n' % (repr(diagnostic_message_success)))
+        if result!=None and result!=(None,None):
+          self._trace_file.write('=> result: %s\n' % (repr(result)))
     return result
 
   def __setattr__(self,name,value):
@@ -242,9 +250,9 @@ class SimpleLDAPObject:
     msgid = self.compare_ext(dn,attr,value,serverctrls,clientctrls)
     try:
       self.result(msgid,all=1,timeout=self.timeout)
-    except _ldap.COMPARE_TRUE:
+    except ldap.COMPARE_TRUE:
       return 1
-    except _ldap.COMPARE_FALSE:
+    except ldap.COMPARE_FALSE:
       return 0
     return None
 
@@ -428,11 +436,11 @@ class SimpleLDAPObject:
     res_type,res_data,res_msgid = self.result2(msgid,all,timeout)
     return res_type,res_data
 
-  def result2(self,msgid=_ldap.RES_ANY,all=1,timeout=None):
+  def result2(self,msgid=ldap.RES_ANY,all=1,timeout=None):
     res_type, res_data, res_msgid, srv_ctrls = self.result3(msgid,all,timeout)
     return res_type, res_data, res_msgid
  
-  def result3(self,msgid=_ldap.RES_ANY,all=1,timeout=None):
+  def result3(self,msgid=ldap.RES_ANY,all=1,timeout=None):
     if timeout is None:
       timeout = self.timeout
     ldap_result = self._ldap_call(self._l.result3,msgid,all,timeout)
@@ -631,7 +639,7 @@ class NonblockingLDAPObject(SimpleLDAPObject):
     self._result_timeout = result_timeout
     SimpleLDAPObject.__init__(self,uri,trace_level,trace_file)
 
-  def result(self,msgid=_ldap.RES_ANY,all=1,timeout=-1):
+  def result(self,msgid=ldap.RES_ANY,all=1,timeout=-1):
     """
     """
     ldap_result = self._ldap_call(self._l.result,msgid,0,self._result_timeout)
@@ -643,7 +651,7 @@ class NonblockingLDAPObject(SimpleLDAPObject):
       while ldap_result[0] is None:
         if (timeout>=0) and (time.time()-start_time>timeout):
           self._ldap_call(self._l.abandon,msgid)
-          raise _ldap.TIMEOUT(
+          raise ldap.TIMEOUT(
             "LDAP time limit (%d secs) exceeded." % (timeout)
           )
         time.sleep(0.00001)
