@@ -3,7 +3,7 @@ ldapurl - handling of LDAP URLs as described in RFC 4516
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: ldapurl.py,v 1.39 2009/07/14 11:03:15 stroeder Exp $
+\$Id: ldapurl.py,v 1.43 2009/08/16 18:45:31 stroeder Exp $
 
 Python compability note:
 This module only works with Python 2.0+ since
@@ -11,7 +11,7 @@ This module only works with Python 2.0+ since
 2. list comprehensions are used.
 """
 
-__version__ = '2.3.9'
+__version__ = '2.3.10'
 
 __all__ = [
   # constants
@@ -102,10 +102,13 @@ class LDAPUrlExtension:
     self.extype = self.extype.strip()
 
   def unparse(self):
-    return '%s%s=%s' % (
-      '!'*(self.critical>0),
-      self.extype,quote(self.exvalue or '')
-    )
+    if self.exvalue is None:
+      return '%s%s' % ('!'*(self.critical>0),self.extype)
+    else:
+      return '%s%s=%s' % (
+        '!'*(self.critical>0),
+        self.extype,quote(self.exvalue or '')
+      )
     
   def __str__(self):
     return self.unparse()
@@ -174,8 +177,9 @@ class LDAPUrlExtensions(UserDict.UserDict):
     
   def parse(self,extListStr):
     for extension_str in extListStr.strip().split(','):
-      e = LDAPUrlExtension(extension_str)
-      self[e.extype] = e
+      if extension_str:
+        e = LDAPUrlExtension(extension_str)
+        self[e.extype] = e
 
   def unparse(self):
     return ','.join([ v.unparse() for v in self.values() ])
@@ -276,7 +280,7 @@ class LDAPUrl:
         rest = rest[qemark_pos:]
       else:
         raise ValueError,'Something completely weird happened!'
-    paramlist=rest.split('?')
+    paramlist=rest.split('?',4)
     paramlist_len = len(paramlist)
     if paramlist_len>=1:
       self.dn = unquote(paramlist[0]).strip()
@@ -295,8 +299,11 @@ class LDAPUrl:
       else:
         self.filterstr = unquote(filterstr)
     if paramlist_len>=5:
-      self.extensions = LDAPUrlExtensions()
-      self.extensions.parse(paramlist[4])
+      if paramlist[4]:
+        self.extensions = LDAPUrlExtensions()
+        self.extensions.parse(paramlist[4])
+      else:
+        self.extensions = None
     return
 
   def applyDefaults(self,defaults):
@@ -378,7 +385,8 @@ class LDAPUrl:
   def __getattr__(self,name):
     if self.attr2extype.has_key(name):
       extype = self.attr2extype[name]
-      if self.extensions.has_key(extype):
+      if self.extensions.has_key(extype) and \
+         not self.extensions[extype].exvalue is None:
         result = unquote(self.extensions[extype].exvalue)
       else:
         return None
