@@ -1,6 +1,6 @@
 /* constants defined for LDAP
  * See http://www.python-ldap.org/ for details.
- * $Id: constants.c,v 1.36 2009/05/04 10:07:52 stroeder Exp $ */
+ * $Id: constants.c,v 1.43 2009/10/23 09:09:37 stroeder Exp $ */
 
 #include "common.h"
 #include "constants.h"
@@ -170,16 +170,21 @@ LDAPinit_constants( PyObject* d )
 	add_int(d,OPT_X_TLS_REQUIRE_CERT);
 	add_int(d,OPT_X_TLS_CIPHER_SUITE);
 	add_int(d,OPT_X_TLS_RANDOM_FILE);
+	add_int(d,OPT_X_TLS_DHFILE);
 	add_int(d,OPT_X_TLS_NEVER);
 	add_int(d,OPT_X_TLS_HARD);
 	add_int(d,OPT_X_TLS_DEMAND);
 	add_int(d,OPT_X_TLS_ALLOW);
 	add_int(d,OPT_X_TLS_TRY);
-        /* only available if OpenSSL supports it => might cause backward compability problems */
+  /* only available if OpenSSL supports it => might cause backward compability problems */
 	add_int(d,OPT_X_TLS_CRLCHECK);
 	add_int(d,OPT_X_TLS_CRL_NONE);
 	add_int(d,OPT_X_TLS_CRL_PEER);
 	add_int(d,OPT_X_TLS_CRL_ALL);
+	add_int(d,OPT_X_TLS_NEWCTX);
+#ifdef OPT_X_TLS_PROTOCOL_MIN
+	add_int(d,OPT_X_TLS_PROTOCOL_MIN);
+#endif
 #endif
 	add_int(d,OPT_X_SASL_MECH);
 	add_int(d,OPT_X_SASL_REALM);
@@ -193,6 +198,12 @@ LDAPinit_constants( PyObject* d )
 #ifdef LDAP_OPT_X_SASL_NOCANON
 	add_int(d,OPT_X_SASL_NOCANON);
 #endif
+#ifdef LDAP_OPT_X_SASL_USERNAME
+	add_int(d,OPT_X_SASL_USERNAME);
+#endif
+#ifdef LDAP_OPT_CONNECT_ASYNC
+	add_int(d,OPT_CONNECT_ASYNC);
+#endif
 
 	add_int(d,DN_FORMAT_LDAP);
 	add_int(d,DN_FORMAT_LDAPV3);
@@ -215,11 +226,11 @@ LDAPinit_constants( PyObject* d )
 	
 	/*add_int(d,OPT_ON);*/
 	obj = PyInt_FromLong(1);
-	PyDict_SetItemString( d, "LDAP_OPT_ON", obj );
+	PyDict_SetItemString( d, "OPT_ON", obj );
 	Py_DECREF(obj);
 	/*add_int(d,OPT_OFF);*/
 	obj = PyInt_FromLong(0);
-	PyDict_SetItemString( d, "LDAP_OPT_OFF", obj );			
+	PyDict_SetItemString( d, "OPT_OFF", obj );			
 	Py_DECREF(obj);
 	
 	add_int(d,OPT_SUCCESS);
