@@ -1,5 +1,5 @@
 /* See http://www.python-ldap.org/ for details.
- * $Id: options.c,v 1.22 2009/05/04 10:07:52 stroeder Exp $ */
+ * $Id: options.c,v 1.33 2009/10/23 09:09:37 stroeder Exp $ */
 
 #include "common.h"
 #include "errors.h"
@@ -13,7 +13,28 @@ set_timeval_from_double( struct timeval *tv, double d ) {
 	tv->tv_sec = (long) floor(d);
 }
 
+/**
+ * Converts a return code from ldap_set_option() or ldap_get_option()
+ * into a python error, and returns NULL.
+ */
+static PyObject *
+option_error(int res, const char *fn)
+{
+    if (res == LDAP_OPT_ERROR)
+        PyErr_SetString(PyExc_ValueError, "option error");
+    else if (res == LDAP_PARAM_ERROR)
+        PyErr_SetString(PyExc_ValueError, "parameter error");
+    else if (res == LDAP_NO_MEMORY) 
+        PyErr_NoMemory();
+    else
+        PyErr_Format(PyExc_SystemError, "error %d from %s", res, fn);
+    return NULL;
+}
 
+/**
+ * Sets an LDAP option.
+ * Returns 0 on failure, 1 on success
+ */
 int
 LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 {
@@ -24,6 +45,7 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     struct timeval tv;
     void *ptr;
     LDAP *ld;
+    LDAPControl **controls = NULL;
 
     ld = self ? self->ldap : NULL;
 
@@ -35,11 +57,14 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 #endif
 	    /* Read-only options */
 	    PyErr_SetString(PyExc_ValueError, "read-only option");
-	    return -1;
+	    return 0;
     case LDAP_OPT_REFERRALS:
     case LDAP_OPT_RESTART:
 #ifdef LDAP_OPT_X_SASL_NOCANON
     case LDAP_OPT_X_SASL_NOCANON:
+#endif
+#ifdef LDAP_OPT_CONNECT_ASYNC
+    case LDAP_OPT_CONNECT_ASYNC:
 #endif
 	    /* Truth-value options */
 	    ptr = PyObject_IsTrue(value) ? LDAP_OPT_ON : LDAP_OPT_OFF;
@@ -51,8 +76,14 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     case LDAP_OPT_PROTOCOL_VERSION:
     case LDAP_OPT_ERROR_NUMBER:
     case LDAP_OPT_DEBUG_LEVEL:
+#ifdef HAVE_TLS
     case LDAP_OPT_X_TLS:
     case LDAP_OPT_X_TLS_REQUIRE_CERT:
+    case LDAP_OPT_X_TLS_NEWCTX:
+#ifdef OPT_X_TLS_PROTOCOL_MIN
+    case LDAP_OPT_X_TLS_PROTOCOL_MIN:
+#endif
+#endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SSF_MIN:
     case LDAP_OPT_X_SASL_SSF_MAX:
@@ -66,12 +97,15 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     case LDAP_OPT_URI:
     case LDAP_OPT_ERROR_STRING:
     case LDAP_OPT_MATCHED_DN:
+#ifdef HAVE_TLS
     case LDAP_OPT_X_TLS_CACERTFILE:
     case LDAP_OPT_X_TLS_CACERTDIR:
     case LDAP_OPT_X_TLS_CERTFILE:
     case LDAP_OPT_X_TLS_KEYFILE:
     case LDAP_OPT_X_TLS_CIPHER_SUITE:
     case LDAP_OPT_X_TLS_RANDOM_FILE:
+    case LDAP_OPT_X_TLS_DHFILE:
+#endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SECPROPS:
 #endif
@@ -94,13 +128,13 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
 	    break;
     case LDAP_OPT_SERVER_CONTROLS:
     case LDAP_OPT_CLIENT_CONTROLS:
-            ptr = List_to_LDAPControls(value);
-            if (ptr == NULL)
-                return -1;
+            if (!LDAPControls_from_object(value, &controls))
+                return 0;
+            ptr = controls;
             break;
     default:
-	    PyErr_SetNone(PyExc_ValueError);
-	    return -1;
+	    PyErr_Format(PyExc_ValueError, "unknown option %d", option);
+	    return 0;
     }
 	
     if (self) LDAP_BEGIN_ALLOW_THREADS(self);
@@ -108,14 +142,14 @@ LDAP_set_option(LDAPObject *self, int option, PyObject *value)
     if (self) LDAP_END_ALLOW_THREADS(self);
 
     if ((option == LDAP_OPT_SERVER_CONTROLS) || (option == LDAP_OPT_CLIENT_CONTROLS))
-        LDAPControl_List_DEL((LDAPControl**) ptr);
+        LDAPControl_List_DEL(controls);
     
     if (res != LDAP_OPT_SUCCESS) {
-	LDAPerr(res);
-	return -1;
+        option_error(res, "ldap_set_option");
+        return 0;
     }
 
-    return 0;
+    return 1;
 }
 
 PyObject *
@@ -123,7 +157,6 @@ LDAP_get_option(LDAPObject *self, int option)
 {
     int res;
     int intval;
-    double doubleval;
     struct timeval *tv;
     LDAPAPIInfo apiinfo;
     LDAPControl **lcs;
@@ -142,7 +175,7 @@ LDAP_get_option(LDAPObject *self, int option)
 	    res = ldap_get_option( ld, option, &apiinfo );
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 	    if (res != LDAP_OPT_SUCCESS)
-		return LDAPerr(res);
+		return option_error(res, "ldap_get_option");
     
 	    /* put the extensions into tuple form */
 	    num_extensions = 0;
@@ -182,46 +215,60 @@ LDAP_get_option(LDAPObject *self, int option)
     case LDAP_OPT_PROTOCOL_VERSION:
     case LDAP_OPT_ERROR_NUMBER:
     case LDAP_OPT_DEBUG_LEVEL:
+#ifdef HAVE_TLS
     case LDAP_OPT_X_TLS:
     case LDAP_OPT_X_TLS_REQUIRE_CERT:
+#ifdef OPT_X_TLS_PROTOCOL_MIN
+    case LDAP_OPT_X_TLS_PROTOCOL_MIN:
+#endif
+#endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SSF_MIN:
     case LDAP_OPT_X_SASL_SSF_MAX:
 #endif
 #ifdef LDAP_OPT_X_SASL_NOCANON
     case LDAP_OPT_X_SASL_NOCANON:
+#endif
+#ifdef LDAP_OPT_CONNECT_ASYNC
+    case LDAP_OPT_CONNECT_ASYNC:
 #endif
 	    /* Integer-valued options */
 	    if (self) LDAP_BEGIN_ALLOW_THREADS(self);
 	    res = ldap_get_option(ld, option, &intval);
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 	    if (res != LDAP_OPT_SUCCESS)
-		return LDAPerr(res);
+		return option_error(res, "ldap_get_option");
 	    return PyInt_FromLong(intval);
 
     case LDAP_OPT_HOST_NAME:
     case LDAP_OPT_URI:
     case LDAP_OPT_ERROR_STRING:
     case LDAP_OPT_MATCHED_DN:
+#ifdef HAVE_TLS
     case LDAP_OPT_X_TLS_CACERTFILE:
     case LDAP_OPT_X_TLS_CACERTDIR:
     case LDAP_OPT_X_TLS_CERTFILE:
     case LDAP_OPT_X_TLS_KEYFILE:
     case LDAP_OPT_X_TLS_CIPHER_SUITE:
     case LDAP_OPT_X_TLS_RANDOM_FILE:
+    case LDAP_OPT_X_TLS_DHFILE:
+#endif
 #ifdef HAVE_SASL
     case LDAP_OPT_X_SASL_SECPROPS:
     case LDAP_OPT_X_SASL_MECH:
     case LDAP_OPT_X_SASL_REALM:
     case LDAP_OPT_X_SASL_AUTHCID:
     case LDAP_OPT_X_SASL_AUTHZID:
+#ifdef LDAP_OPT_X_SASL_USERNAME
+    case LDAP_OPT_X_SASL_USERNAME:
+#endif
 #endif
 	    /* String-valued options */
 	    if (self) LDAP_BEGIN_ALLOW_THREADS(self);
 	    res = ldap_get_option(ld, option, &strval);
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 	    if (res != LDAP_OPT_SUCCESS)
-		return LDAPerr(res);
+		return option_error(res, "ldap_get_option");
 	    if (strval == NULL) {
 		Py_INCREF(Py_None);
 		return Py_None;
@@ -237,7 +284,7 @@ LDAP_get_option(LDAPObject *self, int option)
 	    res = ldap_get_option(ld, option, &tv);
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 	    if (res != LDAP_OPT_SUCCESS)
-		return LDAPerr(res);
+		return option_error(res, "ldap_get_option");
 	    if (tv == NULL) {
 		Py_INCREF(Py_None);
 		return Py_None;
@@ -255,7 +302,7 @@ LDAP_get_option(LDAPObject *self, int option)
 	    if (self) LDAP_END_ALLOW_THREADS(self);
 
 	    if (res != LDAP_OPT_SUCCESS)
-		return LDAPerr(res);
+		return option_error(res, "ldap_get_option");
 
             if (lcs == NULL)
                 return PyList_New(0);
@@ -281,7 +328,7 @@ LDAP_get_option(LDAPObject *self, int option)
             return v;
             
     default:
-	    PyErr_SetObject(PyExc_ValueError, Py_None);
+	    PyErr_Format(PyExc_ValueError, "unknown option %d", option);
 	    return NULL;
     }
 }
